(********************************************************)
(*                                                      *)
(*   Modul:         SUCHE.INC                           *)
(*   Projekt:       CHESS                               *)
(*   Autor:         Michael Thielscher                  *)
(*   erstellt am:   17.04.89                            *)
(*                                                      *)
(********************************************************)


CONST  WunschWert : ARRAY [AufgabenTyp] OF INTEGER = (+2, +1, +2, +1, -2, -1);

VAR  Stellung    : StellungsTyp;
     AufgabenArt : AufgabenTyp;
     TraceOn     : BOOLEAN;
     MaxTiefe, i : BYTE;
     BesterZug   : ARRAY [0..9] OF ZugTyp;


FUNCTION SGN (i: INTEGER): INTEGER;
  BEGIN
    IF i = 0 THEN SGN := 0
    ELSE IF i > 0 THEN SGN := +1
    ELSE SGN := -1
  END; (* SGN *)

PROCEDURE WriteFigur (Feld: FeldTyp; Figur: FigurenTyp);
  BEGIN
    GotoXY(4+4*(Feld MOD 10), 22-2*(Feld DIV 10));  Write(FigurenZeichen[Figur])
  END; (* WriteFigur *)

PROCEDURE ShowSquare (Feld: FeldTyp);
  BEGIN
    Write(CHR(Feld MOD 10 + ORD('A') - 1), CHR(Feld DIV 10 + ORD('0') - 1))
  END; (* ShowSquare *)

FUNCTION Weiss_im_Schach (VAR Stellung: StellungsTyp; Feld: FeldTyp): BOOLEAN;
  VAR i: 1..8;
      F: FeldTyp;
  BEGIN
    WITH Stellung DO BEGIN
      IF Feld = 0 THEN BEGIN
        Feld := 21;
        WHILE Brett[Feld] <> W_Koenig DO Feld := SUCC(Feld)
      END; (* then *)
      Weiss_im_Schach := TRUE;
      FOR i := 1 TO 8 DO BEGIN
        IF Brett[Feld+K_Weite[i]] = S_Koenig THEN Exit;
        IF Brett[Feld+S_Weite[i]] = S_Springer THEN Exit;
        F := Feld;
        REPEAT
          F := F + K_Weite[i];
          IF Brett[F] = S_Dame THEN Exit;
          IF (i <= 4) AND (Brett[F] = S_Turm) THEN Exit;
          IF (i >= 5) AND (Brett[F] = S_Laeufer) THEN Exit
        UNTIL Brett[F] <> Leer
      END; (* for *)
      IF (Brett[Feld+9] = S_Bauer) OR (Brett[Feld+11] = S_Bauer) THEN Exit;
      Weiss_im_Schach := FALSE
    END (* with *)
  END; (* Weiss_im_Schach *)

FUNCTION Schwarz_im_Schach (VAR Stellung: StellungsTyp; Feld: FeldTyp): BOOLEAN;
  VAR i: 1..8;
      F: FeldTyp;
  BEGIN
    WITH Stellung DO BEGIN
      IF Feld = 0 THEN BEGIN
        Feld := 21;
        WHILE Brett[Feld] <> S_Koenig DO Feld := SUCC(Feld)
      END; (* then *)
      Schwarz_im_Schach := TRUE;
      FOR i := 1 TO 8 DO BEGIN
        IF Brett[Feld+K_Weite[i]] = W_Koenig THEN Exit;
        IF Brett[Feld+S_Weite[i]] = W_Springer THEN Exit;
        F := Feld;
        REPEAT
          F := F + K_Weite[i];
          IF Brett[F] = W_Dame THEN Exit;
          IF (i <= 4) AND (Brett[F] = W_Turm) THEN Exit;
          IF (i >= 5) AND (Brett[F] = W_Laeufer) THEN Exit
        UNTIL Brett[F] <> Leer
      END; (* for *)
      IF (Brett[Feld-9] = W_Bauer) OR (Brett[Feld-11] = W_Bauer) THEN Exit;
      Schwarz_im_Schach := FALSE
    END (* with *)
  END; (* Schwarz_im_Schach *)

FUNCTION ZugAusfuehren (VAR Stellung: StellungsTyp; VAR Zug: ZugTyp): BOOLEAN;
  BEGIN
    WITH Stellung DO
      WITH Zug DO BEGIN
        IF NOT Schlagzug THEN
          IF Rochade = 0 THEN BEGIN
            IF NeueFigur <> Leer THEN Brett[nach] := NeueFigur
                                 ELSE Brett[nach] := Brett[von];
            Brett[von] := Leer;
            IF TraceOn THEN BEGIN
              WriteFigur(von, Leer);  WriteFigur(nach, Brett[nach])
            END (* then *)
          END (* then *)
          ELSE
            CASE Rochade OF
              1: BEGIN
                   Brett[25] := Leer;  Brett[23] := W_Koenig;
                   Brett[21] := Leer;  Brett[24] := W_Turm;
                   ZugAusfuehren := (NOT Weiss_im_Schach(Stellung, 23)) AND (NOT Weiss_im_Schach(Stellung, 24))
                                                                        AND (NOT Weiss_im_Schach(Stellung, 25));
                   Rochaden[1] := FALSE;  Rochaden[2] := FALSE;
                   en_pass := 0;  am_Zug := NOT am_Zug;
                   IF TraceOn THEN BEGIN
                     WriteFigur(25, Leer);  WriteFigur(23, W_Koenig);
                     WriteFigur(21, Leer);  WriteFigur(24, W_Turm)
                   END; (* then *)
                   Exit
                 END; (* 1 *)
              2: BEGIN
                   Brett[25] := Leer;  Brett[27] := W_Koenig;
                   Brett[28] := Leer;  Brett[26] := W_Turm;
                   ZugAusfuehren := (NOT Weiss_im_Schach(Stellung, 25)) AND (NOT Weiss_im_Schach(Stellung, 26))
                                                                        AND (NOT Weiss_im_Schach(Stellung, 27));
                   Rochaden[1] := FALSE;  Rochaden[2] := FALSE;
                   en_pass := 0;  am_Zug := NOT am_Zug;
                   IF TraceOn THEN BEGIN
                     WriteFigur(25, Leer);  WriteFigur(27, W_Koenig);
                     WriteFigur(28, Leer);  WriteFigur(26, W_Turm)
                   END; (* then *)
                   Exit
                 END; (* 2 *)
              3: BEGIN
                   Brett[95] := Leer;  Brett[93] := S_Koenig;
                   Brett[91] := Leer;  Brett[94] := S_Turm;
                   ZugAusfuehren := (NOT Schwarz_im_Schach(Stellung, 93)) AND (NOT Schwarz_im_Schach(Stellung, 94))
                                                                          AND (NOT Schwarz_im_Schach(Stellung, 95));
                   Rochaden[3] := FALSE;  Rochaden[4] := FALSE;
                   en_pass := 0;  am_Zug := NOT am_Zug;
                   IF TraceOn THEN BEGIN
                     WriteFigur(95, Leer);  WriteFigur(93, S_Koenig);
                     WriteFigur(91, Leer);  WriteFigur(94, S_Turm)
                   END; (* then *)
                   Exit
                 END; (* 3 *)
              4: BEGIN
                   Brett[95] := Leer;  Brett[97] := S_Koenig;
                   Brett[98] := Leer;  Brett[96] := S_Turm;
                   ZugAusfuehren := (NOT Schwarz_im_Schach(Stellung, 95)) AND (NOT Schwarz_im_Schach(Stellung, 96))
                                                                          AND (NOT Schwarz_im_Schach(Stellung, 97));
                   Rochaden[3] := FALSE;  Rochaden[4] := FALSE;
                   en_pass := 0;  am_Zug := NOT am_Zug;
                   IF TraceOn THEN BEGIN
                     WriteFigur(95, Leer);  WriteFigur(97, S_Koenig);
                     WriteFigur(98, Leer);  WriteFigur(96, S_Turm)
                   END; (* then *)
                   Exit
                 END (* 4 *)
            END (* case *)
        ELSE BEGIN
          IF NeueFigur <> Leer THEN Brett[nach] := NeueFigur
                               ELSE Brett[nach] := Brett[von];
          Brett[von] := Leer;
          IF TraceOn THEN BEGIN
            WriteFigur(von, Leer);  WriteFigur(nach, Brett[nach])
          END; (* then *)
          IF ep > 0 THEN BEGIN
            Brett[nach + 10*SGN(von-nach)] := Leer;
            IF TraceOn THEN WriteFigur(nach + 10*SGN(von-nach), Leer)
          END (* then *)
        END; (* else *)
        IF (ABS(von-nach) = 20) AND (Brett[nach] IN [W_Bauer, S_Bauer]) THEN en_pass := von MOD 10
                                                                        ELSE en_pass := 0;
        IF (von = 25) OR (nach = 25) THEN BEGIN
          Rochaden[1] := FALSE;  Rochaden[2] := FALSE
        END (* then *)
        ELSE BEGIN
          IF (von = 21) OR (nach = 21) THEN Rochaden[1] := FALSE;
          IF (von = 28) OR (nach = 28) THEN Rochaden[2] := FALSE
        END; (* else *)
        IF (von = 95) OR (nach = 95) THEN BEGIN
          Rochaden[3] := FALSE;  Rochaden[4] := FALSE
        END (* then *)
        ELSE BEGIN
          IF (von = 91) OR (nach = 91) THEN Rochaden[3] := FALSE;
          IF (von = 98) OR (nach = 98) THEN Rochaden[4] := FALSE
        END; (* else *)
        IF am_Zug THEN ZugAusfuehren := NOT Weiss_im_Schach(Stellung, 0)
                  ELSE ZugAusfuehren := NOT Schwarz_im_Schach(Stellung, 0);
        am_Zug := NOT am_Zug
      END (* with *)
  END; (* ZugAusfuehren *)

PROCEDURE ZugZuruecknehmen (VAR Stellung: StellungsTyp; VAR Zug: ZugTyp);
  BEGIN
    WITH Stellung DO
      WITH Zug DO BEGIN
        am_Zug := NOT am_Zug;
        IF NOT Schlagzug THEN
          IF Rochade = 0 THEN BEGIN
            IF NeueFigur <> Leer THEN
              IF am_Zug THEN Brett[von] := W_Bauer ELSE Brett[von] := S_Bauer
            ELSE Brett[von] := Brett[nach];
            Brett[nach] := Leer;
            IF TraceOn THEN BEGIN
              WriteFigur(von, Brett[von]);  WriteFigur(nach, Leer)
            END (* then *)
          END (* then *)
          ELSE
            CASE Rochade OF
              1: BEGIN
                   Brett[25] := W_Koenig;  Brett[23] := Leer;
                   Brett[21] := W_Turm;    Brett[24] := Leer;
                   IF TraceOn THEN BEGIN
                     WriteFigur(25, W_Koenig);  WriteFigur(23, Leer);
                     WriteFigur(21, W_Turm);    WriteFigur(24, Leer)
                   END (* then *)
                 END; (* 1 *)
              2: BEGIN
                   Brett[25] := W_Koenig;  Brett[27] := Leer;
                   Brett[28] := W_Turm;    Brett[26] := Leer;
                   IF TraceOn THEN BEGIN
                     WriteFigur(25, W_Koenig);  WriteFigur(27, Leer);
                     WriteFigur(28, W_Turm);    WriteFigur(26, Leer)
                   END (* then *)
                 END; (* 2 *)
              3: BEGIN
                   Brett[95] := S_Koenig;  Brett[93] := Leer;
                   Brett[91] := S_Turm;    Brett[94] := Leer;
                   IF TraceOn THEN BEGIN
                     WriteFigur(95, S_Koenig);  WriteFigur(93, Leer);
                     WriteFigur(91,S_Turm);     WriteFigur(94, Leer)
                   END (* then *)
                 END; (* 3 *)
              4: BEGIN
                   Brett[95] := S_Koenig;  Brett[97] := Leer;
                   Brett[98] := S_Turm;    Brett[96] := Leer;
                   IF TraceOn THEN BEGIN
                     WriteFigur(95, S_Koenig);  WriteFigur(97, Leer);
                     WriteFigur(98, S_Turm);    WriteFigur(96, Leer)
                   END (* then *)
                 END (* 4 *)
            END (* case *)
        ELSE BEGIN
          IF NeueFigur <> Leer THEN
            IF am_Zug THEN Brett[von] := W_Bauer ELSE Brett[von] := S_Bauer
          ELSE Brett[von] := Brett[nach];
          IF TraceOn THEN WriteFigur(von, Brett[von]);
          IF ep > 0 THEN BEGIN
            Brett[nach] := Leer;  Brett[nach + 10*SGN(von-nach)] := hit;
            IF TraceOn THEN BEGIN
              WriteFigur(nach, Leer);  WriteFigur(nach + 10*SGN(von-nach), hit)
            END (* then *)
          END (* then *)
          ELSE BEGIN
            Brett[nach] := hit;
            IF TraceOn THEN WriteFigur(nach, hit)
          END (* else *)
        END (* else *)
      END (* with *)
  END; (* ZugZuruecknehmen *)

FUNCTION ErmittleStellungswert (Tiefe: BYTE): INTEGER;

  VAR  ZugListe              : ZuglistenTyp;
       ZugZahl, ZugNr        : BYTE;
       Wert                  : INTEGER;
       KeinLegalerZug, Legal : BOOLEAN;
       MEMRochaden           : RochadenTyp;
       MEMen_pass            : 0..9;

  BEGIN
    IF Tiefe > MaxTiefe THEN  ErmittleStellungswert := 0
    ELSE
      WITH Stellung DO BEGIN
        Zuggenerator(Stellung, ZugListe, ZugZahl);
        KeinLegalerZug := TRUE;
        FOR ZugNr := 1 TO ZugZahl DO BEGIN
          MEMRochaden := Rochaden;  MEMen_pass := en_pass;
          Legal := ZugAusfuehren(Stellung, ZugListe[ZugNr]);
          IF Legal THEN BEGIN
            KeinLegalerZug := FALSE;
            Wert := ErmittleStellungswert(Tiefe+1);
            ZugZuruecknehmen(Stellung, ZugListe[ZugNr]);
            IF am_Zug THEN BEGIN
              IF Wert = WunschWert[AufgabenArt] THEN BEGIN
                ErmittleStellungswert := Wert;
                BesterZug[Tiefe] := ZugListe[ZugNr];  Exit
              END (* then *)
            END (* then *)
            ELSE
              IF AufgabenArt IN [Hilfsmatt, Hilfspatt] THEN BEGIN
                IF Wert = WunschWert[AufgabenArt] THEN BEGIN
                  ErmittleStellungswert := Wert;
                  BesterZug[Tiefe] := ZugListe[ZugNr];  Exit
                END (* then *)
              END (* then *)
              ELSE
                IF Wert <> WunschWert[AufgabenArt] THEN BEGIN
                  ErmittleStellungswert := Wert;  Exit
                END (* then *)
          END (* then *)
          ELSE  ZugZuruecknehmen(Stellung, ZugListe[ZugNr]);
          Rochaden := MEMRochaden;  en_pass := MEMen_pass
        END; (* for *)
        IF KeinLegalerZug THEN
          IF am_Zug THEN
            IF Weiss_im_Schach(Stellung, 0) THEN ErmittleStellungswert := -2
                                            ELSE ErmittleStellungswert := -1
          ELSE
            IF Schwarz_im_Schach(Stellung, 0) THEN ErmittleStellungswert := +2
                                              ELSE ErmittleStellungswert := +1
        ELSE ErmittleStellungswert := Wert
      END (* with *)
  END; (* ErmittleStellungswert *)

PROCEDURE Suche;
  VAR ch  : CHAR;
      Wert: INTEGER;
  BEGIN
    StellungsEingabe(Stellung);
    StellungsAusgabe(Stellung);
    GotoXY(48,  4);  Write('1 = Matt in n Zgen');
    GotoXY(48,  5);  Write('2 = Patt in n Zgen');
    GotoXY(48,  7);  Write('3 = Hilfsmatt in n Zgen');
    GotoXY(48,  8);  Write('4 = Hilfspatt in n Zgen');
    GotoXY(48, 10);  Write('5 = Selbstmatt in n Zgen');
    GotoXY(48, 11);  Write('6 = Selbstpatt in n Zgen');
    GotoXY(48, 16);  Write('Ihre Wahl (1-6) ?  ');
    REPEAT  Read(Kbd, ch)  UNTIL ch IN ['1'..'6'];
    Write(ch);
    CASE ch OF
      '1': AufgabenArt := Matt;
      '2': AufgabenArt := Patt;
      '3': AufgabenArt := Hilfsmatt;
      '4': AufgabenArt := Hilfspatt;
      '5': AufgabenArt := Selbstmatt;
      '6': AufgabenArt := Selbstpatt
    END; (* case *)
    Stellung.am_Zug := NOT (AufgabenArt IN [Hilfsmatt, Hilfspatt]);
    StellungsAusgabe(Stellung);
    GotoXY(1, 23);  Write('Zahl der Zge n (1-5) ?  ');
    REPEAT  Read(Kbd, ch)  UNTIL ch IN ['1'..'5'];
    Write(ch);
    IF AufgabenArt IN [Matt, Patt] THEN MaxTiefe := 2*(ORD(ch)-ORD('0')) - 1
                                   ELSE MaxTiefe := 2*(ORD(ch)-ORD('0'));
    GotoXY(1, 24);  Write('Trace on');  TraceOn := YesNo;
    Write('            Der Computer arbeitet. Bitte warten ...');
    Wert := ErmittleStellungswert(0);
    GotoXY(54, 2);
    IF Wert = WunschWert[AufgabenArt] THEN BEGIN
      Write('Lsung gefunden!');
      IF NOT (AufgabenArt IN [Hilfsmatt, Hilfspatt]) THEN BEGIN
        GotoXY(60, 5);
        WITH BesterZug[0] DO BEGIN
          ShowSquare(von);  Write(' - ');  ShowSquare(nach)
        END (* with *)
      END (* then *)
      ELSE
        FOR i := 0 TO MaxTiefe-1 DO BEGIN
          GotoXY(60, 5+i);
          WITH BesterZug[i] DO BEGIN
            ShowSquare(von);  Write(' - ');  ShowSquare(nach)
          END (* with *)
        END (* for *)
    END (* then *)
    ELSE Write('Keine Lsung gefunden!')
  END; (* Suche *)
