program Spieltheorie_Graphik;

(*   erstellt die Graphik fuer die Loesung von 2xn-Matrix-Spielen.   *)

{$I GRSYS.INC}

(*
   GRSYS.INC enthaelt folgende Befehle:

    EnterGraphic                       - schaltet Graphikmodus ein
    LeaveGraphic                       - schaltet Graphikmodus aus
    Plot (x, y: integer)               - setzt einen Punkt (x,y)
    Line (x1, y1, x2, y2: integer)     - zeichnet eine Line (x1,y1)-(x2,y2)
    WriteChar (x, y: integer; c: char) - Malt das Zeichen c an die Stelle (x,y)

   und die Konstanten    x_max   - Anzahl der Pixel in x-Richtung
                         y_max   - Anzahl der Pixel in y-Richtung

   (0,0) ist die linke, untere Bildschirmecke
*)

  const  n_max  = 6;                (* maximale Spaltenzahl *)
         Breite = 400;              (* Ausdehnung der Abszisse *)

  type  Matrix = record
                   Spalten : byte;                          (* Spaltenzahl *)
                   Element : array [1..2,1..n_max] of real  (* 2xn-Matrix  *)
                 end;

  var  A      : Matrix;
       i, j   : byte;
       max    : real;
       x0, y0 : integer;
       dy,
       e1, e2 : real;
       s      : string[20];

  procedure Eingabe;                 (* Eingabe der Matrix A *)
    var i, j: byte;
    begin
      ClrScr;
      with A do
        begin
          Write('Anzahl der Spalten: '); ReadLn(Spalten); WriteLn;
          for i := 1 to 2 do
            begin
              for j := 1 to Spalten do
                begin
                  Write(i,'. Zeile, ',j,'. Spalte : '); ReadLn(Element[i,j])
                end;
              WriteLn
            end
        end
    end;

  procedure Koordinatensystem;         (* zeichnet das Koordinatensystem *)
    const s : string[11] =  'Strategie 1';
    var i: byte;
        x: integer;
    begin
      Line(x0, y0, x0+Breite, y0);            (* die Abszisse *)
      Line(x0, 0, x0, y_max);                 (* die linke  Ordinate *)
      Line(x0+Breite, 0, x0+Breite, y_max);   (* die rechte Ordinate *)

      for i := 1 to 9 do                      (* die Skalierung *)
        begin
          x := x0 + i*(Breite div 10);
          Line(x, y0+4, x, y0-4);
          WriteChar(x-12, y0-8, '0'); WriteChar(x-4, y0-8, '.');
          WriteChar(x+4, y0-8, Chr(Ord('0')+i))
        end;

      for i := 1 to Length(s) do                (* Ausgabe des Strings s *)
        WriteChar(x0-8*Length(s)+8*(i-2), y0-16, s[i])
    end;

  begin     { Hauptprogramm }
    Eingabe;
    with A do
      begin
        EnterGraphic;
        max := 0.0;       (* Bestimmung der maximalen Ausdehnung der
                             Ordinate (maximales Matrixelement)      *)
        for i := 1 to 2 do
          for j := 1 to Spalten do
            if Abs(Element[i,j]) > max then
              max := Abs(Element[i,j]);
        max := 1.1*max;                       (* Stauchung, um nicht
                                                 an den Rand zu stossen *)
        x0 := x_max div 2 - Breite div 2;     (* Ursprung des Systems *)
        y0 := y_max div 2;
        dy := (y_max div 2) / max;            (* Anzahl der Bildschirmpunkte,
                                                 die der Einheitslaenge auf
                                                 der Ordinate entspricht.   *)
        Koordinatensystem;
        for i := 1 to Spalten do
          begin
            e1 := Element[1,i]; e2 := Element[2,i];
            Line(x0, Round(y0 + e2*dy), x0+400, y0 + Round(e1*dy));

            Str(e2:2:2, s);                   (* Wert der Ordinate fuer x0 *)
            for j := 1 to Length(s) do
              WriteChar(x0-4-8*(Length(s)-j+1), y0+Round(e2*dy)+4, s[j]);

            Str(e1:2:2,s);                    (* Wert der Ordinate fuer x=1 *)
            for j := 1 to Length(s) do
              WriteChar(x0+Breite+4+8*(j-1), y0+Round(e1*dy)+4, s[j])
          end;
        repeat until KeyPressed;
        LeaveGraphic
      end
  end.
